/* 
Teknisk fysik, Umeå: Robottävlingen 2014
This code was written for use with Processing v2.1 
using the proCONTROLL library available from 
http://creativecomputing.cc/p5libs/procontroll/controllstick_class_controllstick.htm 

Courtesy of 2013 year's finalists team LagET as well as 
this year's finalists team Bumblebee ;] Happy coding!
*/

/*
This code can be used for interpreting data to be sent to the 
Arduino from the computer. It takes the input from the controller 
and sends an array of bytes out whichever port your Arduino is 
connected to. A handshake byte preceeds the byte array containing the useful data. 
An arduino code example is provided at the bottom of this sketch.

Recommended configuration:

L1: decreases the maximum speed
R1: increases the maximum speed
Left analog stick: controls the left tread
Right analog stick: controls the right tread
cooliehat(the big green button): controls two servos, 
                                 one in Y-direction and one in the X-direction
*/


import procontroll.*;        //controller feed
import java.io.*;            //internal dependency for procontroll
import processing.serial.*;  //for communication via Serial
import processing.net.*;  //Imports necessary network libraries
Client myClient;          //Declares an instant myClient of type Client
int dataIn;               //Declares integer

Serial myPort;               // for communication with arduino

ControllIO controll;         // abstraction of the controller object
ControllDevice device;       // abstraction of entire controller
ControllStick stick;         // left analog stick
ControllStick stick2;        // right analog stick
ControllButton triangle;     // triangle
ControllButton circle;       // circle
ControllButton cross;        // cross
ControllButton square;       // square
ControllButton L2;           // L2 
ControllButton R2;           // R2 
ControllButton L1;           // L1
ControllButton R1;           // R1
ControllButton select;       // select
ControllButton start;        // start
ControllCoolieHat cooliehat; // the cooliehat (big green button)

float cooliehatX = 0;
float cooliehatY = 0;
int posY;
int pos2Y;
int posX;
int pos2X;
byte buttons = 0;
byte LR12 = 0;
byte cooliehatVal = 0;
byte[] send = new byte[7];

char handshake = '#'; //Start byte for stable serial communication

void setup(){
  //myClient = new Client(this,"192.168.0.1",5204);  //Initiates a connection to specified ip address through specified port 

  controll = ControllIO.getInstance(this);  
  controll.printDevices();        // print available proCONTROLL devices
  device = controll.getDevice(0); // on a laptop, generally port 0=mousepad,port 1=tangentbord
                                  // port 2=hopefully your controller
  
  // print all available sticks, sliders and buttons of the controller
  device.printSticks();
  device.printSliders();
  device.printButtons();
  
  // set average tolerance, to reduce jerks
  device.setTolerance(0.05f);
  
  // initating the x and y sliders which make up the left and right analog sticks
  //Ändrade siffrorna i getslider.
  ControllSlider sliderX = device.getSlider(0);
  ControllSlider sliderY = device.getSlider(1);
  ControllSlider slider2X = device.getSlider(3);
  ControllSlider slider2Y = device.getSlider(4);

  // initiate the stick with the two sliders previously created
  stick = new ControllStick(sliderX,sliderY);
  stick2 = new ControllStick(slider2X,slider2Y);
  
  // initiate the button objects
  triangle = device.getButton(1);
  circle = device.getButton(2);
  cross = device.getButton(3);
  square = device.getButton(4);
  L2 = device.getButton(5);
  R2 = device.getButton(6);
  L1 = device.getButton(7);
  R1 = device.getButton(8);
  select = device.getButton(9);
  start = device.getButton(10);
  cooliehat = device.getCoolieHat(12);
  
  // initiate the Serial object to connect to the Arduino
  println(Serial.list()); // checks which serial ports are being used
  String portName = "/dev/ttyACM0"; // change depending on which Serial Port your Arduino is connected to
  myPort = new Serial(this,portName,9600); 
  
  
}


void draw(){
  // Set output for cooliehat, -1, 0, 1
  cooliehatX = cooliehat.getX();
  cooliehatY = cooliehat.getY();
  
  
 /*
 Hint: combinations of buttons is possible
 Idle    0
 Left    2
 Right   4
 Up      8
 Down    16  
 */
  cooliehatVal = 0;
  if(cooliehatX < 0){
    cooliehatVal += 2;  
  }
  else if(cooliehatX > 0){
    cooliehatVal += 4; 
  }
  
  if(cooliehatY < 0){
    cooliehatVal += 8;  
  }
  else if(cooliehatY > 0){
    cooliehatVal += 16; 
  }
  
  // sets the output on each analog stick ranging from -127 to 127
  posY = (int)(stick.getY()*127);
  pos2Y = (int)(stick2.getY()*127);
  posX = (int)(stick.getX()*127);
  pos2X = (int)(stick2.getX()*127);
  
  posY = byte(constrain(posY, -127, 127));
  pos2Y = byte(constrain(pos2Y, -127, 127)); 
  posX = byte(constrain(posX, -127, 127));
  pos2X = byte(constrain(pos2X, -127, 127)); 
  
  // Set values for L and R buttons
  /*
  Hint: combinations of buttons is possible
  Idle  0
  L1    2
  L2    4
  R1    8
  R2    16
  */
  LR12 = 0;
  if (L1.pressed()){
    LR12 += 2;
  }
  if (L2.pressed()){
    LR12 += 4;
  }
  if (R1.pressed()){
    LR12 += 8;
  }
  if (R2.pressed()){
    LR12 += 16;
  }

  // action buttons: pewpew lazers and stuff
  
  /*
  Hint: combinations of buttons is possible
  Idle      0
  Triangle  2
  Circle    4
  Cross     8
  Square    16
  Select    32
  Start     64
  */
  
  buttons = 0;
  if (triangle.pressed())      {
    buttons += 2;                   // (0)0000010 
  }if (circle.pressed())  {
   buttons += 4;                    // (0)0000100
  }if (cross.pressed())   {
    buttons += 8;                   // (0)0001000
  }if (square.pressed())  {
    buttons += 16;                  // (0)0010000
  }if (select.pressed())  {
    buttons += 32;                  // (0)0100000
  }if (start.pressed())   {
    buttons += 64;                  // (0)1000000
  }
  
// Allocating an array of bytes for convenient output-transmission
// A byte-datatype is 8 bits of information storing numerical values from -128 to 127
// Hint: In arduino a char can be used for storing numerical values from -128 to 127 
// Hint: Multiple buttons can be combined to give more flexibility but this is a
//       double edged sword since it needs to be kept in mind when coding in arduino. 

    
    send[0] = (byte)(posY);        // data from Y-axis of the left analog stick, range=[-127,127]
    send[1] = (byte)((pos2Y));       // data from Y-axis of the right analog stick, range=[-127,127]
    send[2] = byte(posX);        // data from X-axis of the left analog stick, range=[-127,127]
    send[3] = (byte)(pos2X);       // data from X-axis of the right analog stick, range=[-127,127]
    send[4] = (byte)(cooliehatVal);  // data from the X-axis of cooliehat, [-1,0,1]
    send[5] = (byte)(buttons);     // data from the four right hand side buttons as well as start and select
    send[6] = (byte)(LR12);     // data from L1 button [0,1]
     
      try  {
        myPort.write(handshake);  // char #, can be used as a "handshake" in arduino to make up for packet losses
        myPort.write(send);    
        println(send);
      }

      catch(Exception ex)  {
        //Hint: if you see this when running processing something might be wrong with your serial connection, check COMport
        println("Exception has been reached"); 
      } 
} 

void scramble(){
  if (myClient.available() > 0) {  //If there are more then zero bytes waiting to be read
    dataIn = myClient.read();    //Read bytes to dataIn
    if(dataIn == 1){ // Scramble
      for(int i = 0; i < 7; i++){
        send[i] = 0;
      }
    }
  }
}

/*
Here follows an example sketch for arduino which can be used as a first building block
The handshake byte sent over serial containing a char # is used to correct an error
which occurs from time to time when sending many bytes over serial. 
Author of this text is of course a genooius, geniouos, genious, ...ett geni...
but can still not explain exactly if this error is caused by packet loss or by magic
but the #-byte gets rid of the effects of the error.

Error description:
Byte array seems to be shifted 1 or a few bytes "sideways" making the buttons swap places
and mess up even the happiest of robots.

The code does this:
Lights the test LED if you move your cooliehat, the big green button.
*/

/*
int noBytes = 7;
char buff[7]; //declare how many bytes we are expecting
int ledPin = 13; 

void setup(void) 
{ 
  pinMode(ledPin, OUTPUT);      // sets the digital pin as output  
  Serial.begin(9600);          //Set Baud Rate
} 
void loop(void) 
{

  if(Serial.available()){
    //the buffer to store the bytes in (char[] or byte[]), the number of bytes to read (int) 
    if(Serial.read() == '#'){
      
      if (Serial.readBytes(buff, noBytes) > 0) { 
         
          if(buff[4]>0){
            digitalWrite(ledPin, HIGH);   // turn the LED on (HIGH is the voltage level)
            delay(10);               // wait for a second
            digitalWrite(ledPin, LOW);    // turn the LED off by making the voltage LOW
            delay(10);               // wait for a second
          }    
       }  
     } 
  }
}
*/

